# -*- coding: utf-8 -*-
##############################################################################
# SUDOKEYS:2014
# sudokeys_bluemind_connector
# Author:YL
# created:01-2014
#
#
#
#
##############################################################################

from tools.translate import _
from osv import osv, fields
from suds.client import Client
from suds import WebFault
import threading
import time
from openerp import pooler, sql_db
from datetime import datetime,timedelta
import logging
_logger = logging.getLogger(__name__)     
logging.basicConfig(level=logging.INFO)
if __debug__:
    logging.getLogger('suds.client').setLevel(logging.DEBUG)
else:
    logging.getLogger('suds.client').setLevel(logging.CRITICAL)     
  
class calendar_type(osv.osv):
    _name="calendar.type"
    _sql_constraints = [
        ('name_uniq', 'unique(model_id)', 'The object must be unique!'),
    ]
    _columns = {
            'model_id': fields.many2one("ir.model", 'Object', required=True),
            'name': fields.char('Name',size=255, required=True, translate=True),
            'model': fields.related('model_id','model',type='char', size=255,string='Model', store=True),
            'date_begin':fields.many2one('ir.model.fields','Date begin'),
            'duration':fields.many2one('ir.model.fields','Duration'),
            'date_end':fields.many2one('ir.model.fields','Date end'),
            'blmind_cal_id': fields.integer('Bluemind calendar ID'),
            }
    def onchange_model_id(self, cr, uid, ids, model_id):
        name = ''
        if model_id:
            name = self.pool.get('ir.model').name_get(cr, uid, [model_id]) #read(cr, uid, [model_id], ['name'])[0]['name']
        return {'value':{'name':_(name[0][1])}}
    def get_accessToken(self, cr, uid, config_data):
        user_access_token = False
        blmind_config = self.pool.get('bluemind.config')
        config_data = blmind_config.get_config_data(cr, uid)
        client = blmind_config.get_client(cr, uid, config_data)
        admin_accessToken = blmind_config.get_accessToken(cr, uid, config_data)
        user_access_token = client.service.sudo(admin_accessToken, config_data['domain_admin'])
        return user_access_token    
    
    def create(self, cr, uid, vals, context=None):       
        blmind_config = self.pool.get('bluemind.config')
        config_data = blmind_config.get_config_data(cr, uid)
        client = blmind_config.get_client(cr, uid, config_data)
        accessToken = self.get_accessToken(cr, uid, config_data)
        try:
            createCalendar = True
            calendars = client.service.listCalendars(accessToken)
            for cal in calendars:
                if cal.label==vals['name']:
                    createCalendar = False
            if createCalendar:
                calendarInfo = client.factory.create('calendarInfo')
                calendarInfo.label = vals['name']
                calendarInfo.ownerId = accessToken.userId
                calendarInfo.type = 'calendar'
                calendarInfo.workingDays = "thu,tue,wed,fri,mon"
                #calendarInfo.mail = client.service.getUserEmail(accessToken)
                #calendarInfo.userCreateId = accessToken.userId
                calendarInfo.dayStart = "8"
                calendarInfo.dayEnd = "18"
                calendarInfoResp = client.service.createCalendar(accessToken,calendarInfo)
                if calendarInfoResp:
                    vals['blmind_cal_id'] = calendarInfoResp.id
                client.service.logout(accessToken)
            else:
                client.service.logout(accessToken)
                raise osv.except_osv(_('Error'),_('A calendar with the same name is already in Bluemind!'))
        except Exception, e:
            client.service.logout(accessToken)
            print e
            _logger.debug(e)
            raise osv.except_osv(_('Error'),_("Can't create the current calendar in BlueMind"))
        return super(calendar_type, self).create(cr, uid, vals, context)

    def write(self, cr, uid, ids, vals, context=None):
        if isinstance(ids, int):
            ids = [ids]
        if ('model_id' in vals and vals['model_id']) or ('name' in vals and vals['name']):
            blmind_config = self.pool.get('bluemind.config')
            config_data = blmind_config.get_config_data(cr, uid)
            client = blmind_config.get_client(cr, uid, config_data)
            accessToken = self.get_accessToken(cr, uid, config_data)
            try:
                for id in ids:
                    blmind_cal_id = self.read(cr, uid, [id], ['blmind_cal_id'])[0]['blmind_cal_id']
                    calendarInfoResp = client.service.getCalendarFromId(accessToken, blmind_cal_id)
                    calendarInfoResp.label = vals['name']
                    calendarInfoUpdated = client.service.updateCalendar(accessToken,calendarInfoResp)
                client.service.logout(accessToken)
            except Exception, e:
                client.service.logout(accessToken)
                _logger.debug(e)
                raise osv.except_osv(_('Error'),_("Can't update the calendar in Bluemind!"))
        return super(calendar_type, self).write(cr, uid, ids, vals, context)
    
    def unlink(self, cr, uid, ids, context=None):
        if isinstance(ids,int):
            ids = [ids]
        if ids:
            blmind_config = self.pool.get('bluemind.config')
            config_data = blmind_config.get_config_data(cr, uid)
            client = blmind_config.get_client(cr, uid, config_data)
            accessToken = self.get_accessToken(cr, uid, config_data)
            try:
                for id in ids:
                    blmind_cal_id = self.read(cr, uid, [id],['blmind_cal_id'])[0]['blmind_cal_id']
                    client.service.deleteCalendar(accessToken,blmind_cal_id)
                client.service.logout(accessToken)
            except Exception, e:
                client.service.logout(accessToken)
                _logger.debug(e)
                raise osv.except_osv(_('Error'),_("Can't delete the calendar in BlueMind"))
        return super(calendar_type, self).unlink(cr, uid, ids, context)
calendar_type()

class res_users(osv.osv):
    _inherit='res.users'
    _columns = {
            'user_calendar_ids': fields.many2many('calendar.type','calendar_user_type_rel_','user_id','calendar_type_id','Calendar Type')
            }
res_users()
   
class bluemind_calendar(osv.osv):
    _name = 'bluemind.calendar'
    _columns = {
            'calendar_type_id': fields.many2one('calendar.type','Calendar Type', required=True,ondelete='cascade'),
            'blmind_event_id':fields.integer('BlueMind event'),
            'oerp_event_id': fields.integer('Event'),
            'blmind_user_id': fields.integer('User created the event in Bluemind')
            }
    
    def _get_cal_accessToken(self, cr, uid, blmind_user, config_data):
        user_access_token = False
        blmind_config = self.pool.get('bluemind.config')
        config_data = blmind_config.get_config_data(cr, uid)
        client = blmind_config.get_client(cr, uid, config_data)
        accessToken = blmind_config.get_accessToken(cr, uid, config_data)
        user_access_token = client.service.sudo(accessToken, blmind_user.name)
        return user_access_token
    
    def _get_blmind_user(self, cr, uid, client, accessToken, blmind_user):
        login = blmind_user.name
        if '@' in login:
            login = login.split('@')[0]
        try:
            userQuery = client.factory.create('userQuery')
            userQuery.login = login
            users = client.service.findUsers(accessToken,userQuery)
            if not users:
                raise osv.except_osv(_('Error'),_("Can't find %s user in BlueMind")%login)
        except WebFault, e:
            _logger.debug(e)
        return users[0]
    
    def _prepare_create_event(self, cr, uid, client, accessToken, vals, event, cal_type, blmind_user):        
        cal_name = cal_type.model
        blmind_cal_id = cal_type.blmind_cal_id
        blmind_user = self._get_blmind_user(cr, uid, client, accessToken, blmind_user)
        if cal_name == 'crm.meeting':
            event.extId = vals['id']
            event.ownerId = blmind_user.id
            event.title = vals['name']
            event.date = datetime.strptime(vals['date'], '%Y-%m-%d %H:%M:%S')
            event.allday = vals['allday']
            event.duration = int(vals['duration']*3600)
            categ_ids = vals['categ_ids']
            if categ_ids:
                category_ids = categ_ids
                for etiqu_name in self.pool.get('crm.meeting.type').read(cr, uid, category_ids,['name']):
                    event.tags.append({'label':etiqu_name['name']})
            if vals['location']:
                event.location = vals['location']
            event.type = 'VEVENT'
            if vals['description']:
                event.description = vals['description']
#            if vals['class']:
#                if vals['class']=='private':
            event.privacy = True
            if vals['show_as']:
                if vals['show_as']=='free':
                    event.opacity = 'TRANSPARENT'
                elif vals['show_as']=='busy':
                    event.opacity = 'OPAQUE'
            if vals['attendee_ids']:
                attendee_ids = vals['attendee_ids']
                for attende in attendee_ids:
                    values = attende[2]
                    atten = client.factory.create('attendee')
                    if 'partner_id' in values and values['partner_id']:
                        blmind_contact_ids = self.pool.get('bluemind.contact').search(cr, uid, [('openerp_partner_id','=',values['partner_id'])])
                        if blmind_contact_ids:
                            blmind_contact_id = self.pool.get('bluemind.contact').read(cr, uid, blmind_contact_ids, ['name'])[0]['name']
                            atten.id = blmind_contact_id
                            if 'email' in values and values['email']:
                                atten.email = values['email']
                            atten.type = 'contact'
                            atten.required = "OPT"
                            atten.state = "NEEDSACTION"
                            atten.notify = True
                            atten.alert = 600
                    elif 'email' in values and values['email']:
                        atten.email = values['email']
                        atten.type = 'contact'
                        atten.required = "OPT"
                        atten.state = "NEEDSACTION"
                        atten.notify = True
                        atten.alert = 600                        
                    event.attendees.append(atten)
            attendee = client.factory.create('attendee')
            calendarInfo = client.service.getCalendarFromId(accessToken, blmind_cal_id)
            attendee.calendarInfo = calendarInfo
            attendee.id = blmind_user.id
            attendee.type = 'calendar'
            attendee.required = "CHAIR"
            attendee.state = "ACCEPTED"
            event.attendees.append(attendee)
        if cal_name == 'project.task':
            event.extId = vals['id']
            event.ownerId = blmind_user.id
            event.title = vals['name']
            event.date = datetime.strptime(vals['date_deadline'], '%Y-%m-%d')
            categ_ids = vals['categ_ids']
            if categ_ids:
                category_ids = categ_ids
                for etiqu_name in self.pool.get('project.category').read(cr, uid, category_ids,['name']):
                    event.tags.append({'label':etiqu_name['name']})
            event.allday = True
            event.type = 'VEVENT'
            if vals['description']:
                event.description = vals['description']
            event.privacy = True
            event.opacity = 'TRANSPARENT'
            if vals['partner_id']:
                atten = client.factory.create('attendee')
                blmind_contact_ids = self.pool.get('bluemind.contact').search(cr, uid, [('openerp_partner_id','=',vals['partner_id'])])
                if blmind_contact_ids:
                    blmind_contact_id = self.pool.get('bluemind.contact').read(cr, uid, blmind_contact_ids, ['name'])[0]['name']
                    atten.id = blmind_contact_id
                    atten.type = 'contact'
                    atten.required = "OPT"
                    atten.state = "NEEDSACTION"
                    atten.notify = True
                    atten.alert = 600                       
                event.attendees.append(atten)
            attendee = client.factory.create('attendee')
            calendarInfo = client.service.getCalendarFromId(accessToken, blmind_cal_id)
            attendee.calendarInfo = calendarInfo
            attendee.id = blmind_user.id
            attendee.type = 'calendar'
            attendee.required = "CHAIR"
            attendee.state = "ACCEPTED"
            event.attendees.append(attendee)
        if cal_name == 'hr.holidays':
            event.extId = vals['id']
            event.ownerId = blmind_user.id
            event.description = self.pool.get('hr.holidays.status').read(cr, uid,[vals['holiday_status_id']],['name'])[0]['name']
            event.title = vals['name']
            event.date = datetime.strptime(vals['date_from'], '%Y-%m-%d %H:%M:%S')
            event.completion = datetime.strptime(vals['date_to'], '%Y-%m-%d %H:%M:%S')
            duration = int(( datetime.strptime(vals['date_to'], '%Y-%m-%d %H:%M:%S') - datetime.strptime(vals['date_from'], '%Y-%m-%d %H:%M:%S')).total_seconds())
            event.duration = duration
            event.type = 'VEVENT'
            event.privacy = True
            event.opacity = 'TRANSPARENT'
#            if vals['partner_id']:
#                atten = client.factory.create('attendee')
#                blmind_contact_ids = self.pool.get('bluemind.contact').search(cr, uid, [('openerp_partner_id','=',vals['partner_id'])])
#                if blmind_contact_ids:
#                    blmind_contact_id = self.pool.get('bluemind.contact').read(cr, uid, blmind_contact_ids, ['name'])[0]['name']
#                    atten.id = blmind_contact_id
#                    atten.type = 'contact'
#                    atten.required = "OPT"
#                    atten.state = "NEEDSACTION"
#                    atten.notify = True
#                    atten.alert = 600                       
#                event.attendees.append(atten)
            attendee = client.factory.create('attendee')
            calendarInfo = client.service.getCalendarFromId(accessToken, blmind_cal_id)
            attendee.calendarInfo = calendarInfo
            attendee.id = blmind_user.id
            attendee.type = 'calendar'
            attendee.required = "CHAIR"
            attendee.state = "ACCEPTED"
            event.attendees.append(attendee)
        print '--'*20
        print event    
        return event

    def _prepare_update_event(self, cr, uid, client, accessToken, vals, event, cal_type):
        
        cal_name = cal_type.model
        blmind_cal_id = cal_type.blmind_cal_id
#        blmind_user = self._get_blmind_user(cr, uid, client, accessToken, blmind_user)
        if cal_name == 'crm.meeting':
            if 'name' in vals:
                event.title = vals['name']
            if 'date' in vals:
                event.date = datetime.strptime(vals['date'], '%Y-%m-%d %H:%M:%S')
            if 'allday' in vals and vals['allday']:
                event.allday = vals['allday']
            if 'duration' in vals:
                event.duration = int(vals['duration']*3600)
            if 'categ_ids' in vals:
                categ_ids = vals['categ_ids']
                if categ_ids and categ_ids[0] and categ_ids[0][2]:
                    event.tags = []
                    category_ids = categ_ids[0][2]
                    for etiqu_name in self.pool.get('crm.meeting.type').read(cr, uid, category_ids,['name']):
                        event.tags.append({'label':etiqu_name['name']})
            if 'location' in vals and vals['location'] :
                event.location = vals['location']
            if 'description' in vals and vals['description']:
                event.description = vals['description']
            if 'show_as' in vals and vals['show_as']:
                if vals['show_as']=='free':
                    event.opacity = 'TRANSPARENT'
                elif vals['show_as']=='busy':
                    event.opacity = 'OPAQUE'
            if 'attendee_ids' in vals and vals['attendee_ids']:
                attendee_ids = vals['attendee_ids']
                for attende in attendee_ids:
                    if attende[0]==0:
                        values = attende[2]
                        atten = client.factory.create('attendee')
                        if 'partner_id' in values and values['partner_id']:
                            blmind_contact_ids = self.pool.get('bluemind.contact').search(cr, uid, [('openerp_partner_id','=',values['partner_id'])])
                            if blmind_contact_ids:
                                blmind_contact_id = self.pool.get('bluemind.contact').read(cr, uid, blmind_contact_ids, ['name'])[0]['name']
                                atten.id = blmind_contact_id
                                if 'email' in values and values['email']:
                                    atten.email = values['email']
                                atten.type = 'contact'
                                atten.required = "OPT"
                                atten.state = "NEEDSACTION"
                                atten.notify = True
                                atten.alert = 600
                        elif 'email' in values and values['email']:
                            atten.email = values['email']
                            atten.type = 'contact'
                            atten.required = "OPT"
                            atten.state = "NEEDSACTION"
                            atten.notify = True
                            atten.alert = 600                        
                        event.attendees.append(atten)
#                    elif attende[0]==1:
#                        values = attende[2]
#                        if 'partner_id' in values:
#                            if values['partner_id']:
#                                blmind_contact_ids = self.pool.get('bluemind.contact').search(cr, uid, [('openerp_partner_id','=',values['partner_id'])])
#                                if blmind_contact_ids:
#                                    blmind_contact_id = self.pool.get('bluemind.contact').read(cr, uid, blmind_contact_ids, ['name'])[0]['name']
#                                    atten.id = blmind_contact_id
#                                    if 'email' in values and values['email']:
#                                        atten.email = values['email']
#                                    atten.type = 'contact'
#                                    atten.required = "OPT"
#                                    atten.state = "NEEDSACTION"
#                                    atten.notify = True
#                                    atten.alert = 600
#                            else:
#                                atten.id = None
#                        elif 'email' in values:
#                            if values['email']:
#                                atten.email = values['email']
#                                atten.type = 'contact'
#                                atten.required = "OPT"
#                                atten.state = "NEEDSACTION"
#                                atten.notify = True
#                                atten.alert = 600   
#                            else:
#                                atten.email = None                     
#                        event.attendees.append(atten)
        if cal_name == 'project.task':
            if 'name' in vals:
                event.title = vals['name']
            if 'date_deadline' in vals:
                event.date = datetime.strptime(vals['date_deadline'], '%Y-%m-%d')
            if 'categ_ids' in vals:
                categ_ids = vals['categ_ids']
                if categ_ids and categ_ids[0] and categ_ids[0][2]:
                    event.tags = []
                    category_ids = categ_ids[0][2]
                    for etiqu_name in self.pool.get('project.category').read(cr, uid, category_ids,['name']):
                        event.tags.append({'label':etiqu_name['name']})
            if 'description' in vals:
                event.description = vals['description']
            if 'partner_id' in vals:
                for attende in event.attendees:
                    if attende.type == 'contact':
                        event.attendees.remove(attende)
                if vals['partner_id']:
                    atten = client.factory.create('attendee')
                    blmind_contact_ids = self.pool.get('bluemind.contact').search(cr, uid, [('openerp_partner_id','=',vals['partner_id'])])
                    if blmind_contact_ids:
                        blmind_contact_id = self.pool.get('bluemind.contact').read(cr, uid, blmind_contact_ids, ['name'])[0]['name']
                        atten.id = blmind_contact_id
                        atten.type = 'contact'
                        atten.required = "OPT"
                        atten.state = "NEEDSACTION"
                        atten.notify = True
                        atten.alert = 600                       
                    event.attendees.append(atten)
        if cal_name == 'hr.holidays':
            if 'holiday_status_id' in vals: 
                event.title = self.pool.get('hr.holidays.status').read(cr, uid,[vals['holiday_status_id']],['name'])[0]['name']
            if 'name' in vals:
                if vals['name']:
                    event.description = vals['name']
                else:
                    event.description = ''
            if 'date_from' in vals or 'date_to' in vals:
                date_from = False
                date_to = False
                duration = False
                if 'date_from' in vals:
                    event.date = datetime.strptime(vals['date_from'], '%Y-%m-%d %H:%M:%S')
                    date_from = event.date
                if 'date_to' in vals:
                    date_to = datetime.strptime(vals['date_to'], '%Y-%m-%d %H:%M:%S')
                if date_from and date_to:
                    duration = int((date_to - date_from).total_seconds())
                if date_from and not date_to:
                    date_to = datetime.strptime(self.pool.get(cal_name).read(cr, uid, [event.extId], ['date_to'])[0]['date_to'], '%Y-%m-%d %H:%M:%S')
                    duration = int((date_to - date_from).total_seconds())
                if date_to and not date_from:
                    date_from = datetime.strptime(self.pool.get(cal_name).read(cr, uid, [event.extId], ['date_from'])[0]['date_from'], '%Y-%m-%d %H:%M:%S')
                    duration = int((date_to - date_from).total_seconds())                    
                event.duration = duration    
#            if vals['partner_id']:
#                atten = client.factory.create('attendee')
#                blmind_contact_ids = self.pool.get('bluemind.contact').search(cr, uid, [('openerp_partner_id','=',vals['partner_id'])])
#                if blmind_contact_ids:
#                    blmind_contact_id = self.pool.get('bluemind.contact').read(cr, uid, blmind_contact_ids, ['name'])[0]['name']
#                    atten.id = blmind_contact_id
#                    atten.type = 'contact'
#                    atten.required = "OPT"
#                    atten.state = "NEEDSACTION"
#                    atten.notify = True
#                    atten.alert = 600                       
#                event.attendees.append(atten)
        return event
    
    def update_events_from_bluemind(self, cr, uid):
        blmind_config = self.pool.get('bluemind.config')
        config_data = blmind_config.get_config_data(cr, uid)
        client = blmind_config.get_client(cr, uid, config_data)
        accessToken = blmind_config.get_accessToken(cr, uid, config_data)
        calendar_config = self.pool.get('calendar.config')
        calendar_config_ids = calendar_config.search(cr, uid, [])
        cal_type = self.pool.get('calendar.type')
        try:
            cal_ids = cal_type.search(cr, uid, [])
            blmind_cals = cal_type.read(cr, uid, cal_ids, ['blmind_cal_id','model'])
            for blmind_cal in blmind_cals:
                blmind_cal_id = blmind_cal['blmind_cal_id']
                syncCalendar = client.factory.create('syncCalendar')
                cal = client.service.getCalendarFromId(accessToken, blmind_cal_id)
                syncCalendar.calendar = cal and cal.id or None
                event_last_sync = calendar_config.read(cr, uid, calendar_config_ids, ['event_last_sync'])[0]['event_last_sync']
                datetime_value = datetime.strptime(event_last_sync,'%Y-%m-%d %H:%M:%S')-timedelta(seconds=1)
                syncCalendar.lastSync = datetime_value
                syncCalendar.mode = 'CHANGED_AFTER_DATE'
                eventChanges = client.factory.create('eventChanges')
                eventChanges.lastSync = datetime_value
                eventChangesResp = client.service.doEventSync(accessToken,syncCalendar, eventChanges,None,None)
                print eventChangesResp
                if hasattr(eventChangesResp, 'removed'):
                    print eventChangesResp.removed
                    event2delete_ids = [int(elm) for elm in eventChangesResp.removed]
                    self.delete_oerp_events(cr, uid, event2delete_ids, blmind_cal)
                if hasattr(eventChangesResp, 'updated'):
                    events = [event for event in eventChangesResp.updated]
                    self.update_oerp_events(cr, uid, events, blmind_cal)     
            print 'ii'*20     
            calendar_config.write(cr, uid, calendar_config_ids, {'event_last_sync':eventChangesResp.lastSync.strftime('%Y-%m-%d %H:%M:%S')})
            client.service.logout(accessToken)
        except WebFault, e:
            client.service.logout(accessToken)
            _logger.debug(e)
        except Exception, ex:
            _logger.debug(ex)
        return True
    def delete_oerp_events(self, cr, uid, event_ids, blmind_cal):
        cal_name = blmind_cal['model'] 
        cal_object = self.pool.get(cal_name)
        blmind_cal_ids = self.search(cr, uid,[('calendar_type_id','=',blmind_cal['id']),('blmind_event_id','in',event_ids)])
        if blmind_cal_ids:
            oerp_event_ids= [data['oerp_event_id'] for data in self.read(cr, uid, blmind_cal_ids, ['oerp_event_id'])]
            if oerp_event_ids:
                is_deleted = cal_object.unlink(cr, uid, oerp_event_ids)
                if is_deleted:
                    self.unlink(cr, uid, blmind_cal_ids)
        return True
    def update_oerp_events(self, cr, uid, events, blmind_cal):
        cal_name = blmind_cal['model'] 
        cal_object = self.pool.get(cal_name)
        if cal_name == 'crm.meeting':
            for event in events:
                if not event.title or not event.date:
                    continue
                vals = {}
                vals['name'] = event.title
                vals['date'] = event.date
                if event.allday:
                    vals['allday'] = event.allday
                    vals['date'] = cal_object.onchange_dates(cr, uid, ids=False, start_date=event.date.strftime('%Y-%m-%d %H:%M:%S'), allday=event.allday)['value']['date']
                    vals['date_deadline'] = datetime.strptime(vals['date'],'%Y-%m-%d %H:%M:%S') + timedelta(hours=24)
                else:
                    vals['duration'] = event.duration/3600.00
                    vals['date_deadline'] = vals['date'] + timedelta(hours=event.duration/3600)
                if hasattr(event,'tags'):
                    blmind_tags = []
                    new_event_tags = []
                    if event.tags:
                        blmind_tags = [tag.label for tag in event.tags]
                    if blmind_tags:
                        for blmind_tag in blmind_tags:
                            event_categ_ids = self.pool.get('crm.meeting.type').search(cr, uid, [('name','=',blmind_tag)])
                            if event_categ_ids:
                                new_event_tags.extend(event_categ_ids)
                            else:
                                event_categ_id = self.pool.get('crm.meeting.type').create(cr, uid,{'name':blmind_tag})
                                if event_categ_id:
                                    new_event_tags.append(event_categ_id)
                        if new_event_tags:        
                            vals['categ_ids'] = [[6,False,new_event_tags]]                
                if event.location:
                    vals['location'] = event.location
                if event.description:
                    vals['description'] = event.description          
                if event.opacity:
                    opacity = event.opacity
                    vals['show_as'] = (opacity=='TRANSPARENT' and 'free') or (opacity=='OPAQUE' and 'busy') 
                vals['class'] =  (event.privacy and 'private') or 'public'
                blmind_user_id = self._get_oerp_user(cr, uid, event.owner)
                if blmind_user_id:
                    user_ids = self.pool.get('res.users').search(cr, uid,[('bluemind_user_id','=',blmind_user_id)])
                    if user_ids:
                        vals['user_id'] = user_ids[0]
                    blmind_cal_ids = self.search(cr, uid,[('calendar_type_id','=',blmind_cal['id']),('blmind_event_id','=',event.id),('blmind_user_id','=',blmind_user_id)])
                    oerp_event_id = False
                    if not blmind_cal_ids:
                        oerp_event_id = cal_object.create(cr, uid, vals, context={'from_blmind':True})
                        if oerp_event_id:
                            self.create(cr, uid, {'calendar_type_id':blmind_cal['id'],'blmind_event_id':event.id, 'blmind_user_id':blmind_user_id,'oerp_event_id':oerp_event_id})
                    else:
                        oerp_event_id = self.read(cr, uid, blmind_cal_ids, ['oerp_event_id'])[0]['oerp_event_id']
                        res_write = cal_object.write(cr, uid, [oerp_event_id], vals, context={'from_blmind':True})
                    if oerp_event_id:
                        if hasattr(event,'attendees') and event.attendees:
                            new_attendees = event.attendees
                            alr_attend_ids = cal_object.read(cr, uid, [oerp_event_id],['attendee_ids'])[0]['attendee_ids']
                            if alr_attend_ids:
                                list2delete = [[2,id_exist,False] for id_exist in alr_attend_ids]
                                cal_object.write(cr, uid, [oerp_event_id], {'attendee_ids':list2delete})
                            list2create = []
                            for atten in new_attendees:
                                if hasattr(atten,'email'):
                                    list2create.append([0,False,{'email':atten.email}])
                            if list2create:
                                cal_object.write(cr, uid, [oerp_event_id], {'attendee_ids':list2create})
        if cal_name == 'project.task':
            for event in events:
                vals = {}
                vals['name'] = event.title
                vals['date_deadline'] = event.date.date()
                if event.description:
                    vals['description'] = event.description   
                if hasattr(event,'tags'):
                    blmind_tags = []
                    new_event_tags = []
                    if event.tags:
                        blmind_tags = [tag.label for tag in event.tags]
                    if blmind_tags:
                        for blmind_tag in blmind_tags:
                            event_categ_ids = self.pool.get('project.category').search(cr, uid, [('name','=',blmind_tag)])
                            if event_categ_ids:
                                new_event_tags.extend(event_categ_ids)
                            else:
                                event_categ_id = self.pool.get('project.category').create(cr, uid,{'name':blmind_tag})
                                if event_categ_id:
                                    new_event_tags.append(event_categ_id)
                        if new_event_tags:        
                            vals['categ_ids'] = [[6,False,new_event_tags]] 
                blmind_user_id = self._get_oerp_user(cr, uid, event.owner)       
                if blmind_user_id:
                    user_ids = self.pool.get('res.users').search(cr, uid,[('bluemind_user_id','=',blmind_user_id)])
                    if user_ids:
                        vals['user_id'] = user_ids[0]
                    blmind_cal_ids = self.search(cr, uid,[('calendar_type_id','=',blmind_cal['id']),('blmind_event_id','=',event.id),('blmind_user_id','=',blmind_user_id)])
                    if not blmind_cal_ids:
                        oerp_event_id = cal_object.create(cr, uid, vals, context={'from_blmind':True})
                        if oerp_event_id:
                            self.create(cr, uid, {'calendar_type_id':blmind_cal['id'],'blmind_event_id':event.id, 'blmind_user_id':blmind_user_id,'oerp_event_id':oerp_event_id})
                    else:
                        oerp_event_id = self.read(cr, uid, blmind_cal_ids, ['oerp_event_id'])[0]['oerp_event_id']
                        res_write = cal_object.write(cr, uid, [oerp_event_id], vals, context={'from_blmind':True})
        
        if cal_name == 'hr.holidays':
            for event in events:
                vals = {}
#                holidays_status_ids = self.pool.get('hr.holidays.status').search(cr, uid, [])
#                vals['holiday_status_id'] = holidays_status_ids[0]
                vals['number_of_days_temp'] = 1.00
                vals['date_from'] = event.date
                vals['date_to'] = event.date+timedelta(seconds=event.duration)
                if hasattr(event,'allday') and event.allday:
                    date_from = event.date.replace(hour=8)
                    vals['date_from'] = date_from
                    vals['date_to'] = date_from + timedelta(hours=10)
                if event.title:
                    vals['name'] = event.title      
                blmind_user_id = self._get_oerp_user(cr, uid, event.owner)       
                if blmind_user_id:
                    user_ids = self.pool.get('res.users').search(cr, uid,[('bluemind_user_id','=',blmind_user_id)])
                    if user_ids:
                        employee_ids = self.pool.get('hr.employee').search(cr, uid,[('user_id','in',user_ids)])
                        if employee_ids:
                            vals['employee_id'] = employee_ids[0]
                    
                    blmind_cal_ids = self.search(cr, uid,[('calendar_type_id','=',blmind_cal['id']),('blmind_event_id','=',event.id),('blmind_user_id','=',blmind_user_id)])
                    if not blmind_cal_ids:
                        holiday_status_ids = self.pool.get('hr.holidays.status').search(cr, uid, [('name','in',['Compensatory Days'])])
                        if holiday_status_ids:
                            vals['holiday_status_id'] = holiday_status_ids[0]
                        else:
                            holiday_status_ids = self.pool.get('hr.holidays.status').search(cr, uid, [])
                            vals['holiday_status_id'] = holiday_status_ids[0]
                        oerp_event_id = cal_object.create(cr, uid, vals, context={'from_blmind':True})
                        if oerp_event_id:
                            self.create(cr, uid, {'calendar_type_id':blmind_cal['id'],'blmind_event_id':event.id, 'blmind_user_id':blmind_user_id,'oerp_event_id':oerp_event_id})
                    else:
                        oerp_event_id = self.read(cr, uid, blmind_cal_ids, ['oerp_event_id'])[0]['oerp_event_id']
                        res_write = cal_object.write(cr, uid, [oerp_event_id], vals, context={'from_blmind':True})
        return True
    
    def _get_oerp_user(self, cr, uid, blmind_login):
        blmind_user_id = False
        bluemind_users = self.pool.get('bluemind.users')        
        blmind_users_ids = bluemind_users.search(cr, uid, [])
        for data in bluemind_users.read(cr, uid, blmind_users_ids,['name']):
            name = data['name']
            if '@' in name:
                name = name.split('@')[0] 
            if blmind_login==name:
                blmind_user_id = data['id']
        if not blmind_user_id:
            admin = self.pool.get('res.users').browse(cr, uid,[1])[0]
            if not admin.bluemind_user_id:
                _logger.debug('Must add a bluemind user to admin user !')
            else:
                blmind_user_id = admin.bluemind_user_id.id
        return blmind_user_id  
         
    def create_event(self, cr, uid, vals, cal_type, blmind_user):
        blmind_config = self.pool.get('bluemind.config')
        config_data = blmind_config.get_config_data(cr, uid)
        client = blmind_config.get_client(cr, uid, config_data)
        accessToken = self._get_cal_accessToken(cr, uid, blmind_user, config_data)
        event_id = False
        try:
            event = client.factory.create('event')
            event = self._prepare_create_event(cr, uid, client, accessToken, vals, event, cal_type, blmind_user)
            event_id = client.service.createEvent(accessToken, event)
            if event_id:
                self.create(cr, uid, {'calendar_type_id':cal_type.id, 'blmind_event_id':event_id,'oerp_event_id':event.extId,'blmind_user_id':blmind_user.id})
            client.service.logout(accessToken)
            cr.commit()
            cr.close()
        except WebFault, e:
            cr.close()
            client.service.logout(accessToken)
            print e
            _logger.debug(e)
        return event_id
   
    def update_event(self, cr, uid, vals, cal_type, blmind_user, event_id):
        blmind_config = self.pool.get('bluemind.config')
        config_data = blmind_config.get_config_data(cr, uid)
        client = blmind_config.get_client(cr, uid, config_data)
        accessToken = self._get_cal_accessToken(cr, uid, blmind_user, config_data)
        eventResp = False
        try:
            event = client.service.getEventFromId(accessToken,event_id)
            event = self._prepare_update_event(cr, uid, client, accessToken, vals, event, cal_type)
            eventResp = client.service.updateEvent(accessToken, event,True) 
            client.service.logout(accessToken) 
            cr.commit()
            cr.close()  
        except WebFault, e:
            cr.close()
            client.service.logout(accessToken)
            print e
            _logger.debug(e)
        return eventResp
    
    def delete_event(self, cr, uid, cal_type, blmind_user, event_id):
        blmind_config = self.pool.get('bluemind.config')
        config_data = blmind_config.get_config_data(cr, uid)
        client = blmind_config.get_client(cr, uid, config_data)
        accessToken = self._get_cal_accessToken(cr, uid, blmind_user, config_data)
        try:
            event = client.service.deleteEvent(accessToken, event_id) 
            client.service.logout(accessToken)
            if event:
                blmind_cal_ids = self.search(cr, uid,[('oerp_event_id','=',event.extId),('calendar_type_id','=',cal_type.id)])
                self.unlink(cr, uid, blmind_cal_ids)
            cr.commit()
            cr.close()
        except WebFault, e:
            cr.close()
            client.service.logout(accessToken)
            print e
            _logger.debug(e)    
        return event
bluemind_calendar()

###########################Calendars to synchronize###########################

class crm_meeting(osv.osv):
    _inherit = 'crm.meeting'  
    def create(self, cr, uid, vals, context=None):
        if not context:
            context = {}
        res = super(crm_meeting, self).create(cr, uid, vals, context)
        if res:
            dbname = cr.dbname
            db = sql_db.db_connect(dbname)
            cr = db.cursor()
            cal_type = self._is_auto_sync(cr, uid, context)
            res_user = self.pool.get('res.users').browse(cr, uid, [uid])[0]
            blmind_user = res_user.bluemind_user_id or False
            if cal_type and blmind_user and not context.get('from_blmind',False):
                vals['id'] = res
                create_event = self.pool.get('bluemind.calendar').create_event
                thr = threading.Thread(target=create_event, args=(cr,uid,vals,cal_type,blmind_user))
                thr.start()
            else:
                cr.close()
        return res
    
    def write(self, cr, uid, ids, vals, context=None):
        if not context:
            context = {}
        res = super(crm_meeting, self).write(cr, uid, ids, vals, context)
        blimind_cal = self.pool.get('bluemind.calendar')
        dbname = cr.dbname
        db = sql_db.db_connect(dbname)
        cr = db.cursor()
        cal_type = self._is_auto_sync(cr, uid, context)
        res_user = self.pool.get('res.users').browse(cr, uid, [uid])[0]
        blmind_user = res_user.bluemind_user_id or False
        if cal_type and blmind_user and not context.get('from_blmind',False):
            blmind_cal_ids = blimind_cal.search(cr, uid,[('oerp_event_id','in',ids),('calendar_type_id','=',cal_type.id)])
            for event in blimind_cal.browse(cr, uid, blmind_cal_ids):
                update_event = blimind_cal.update_event
                thr = threading.Thread(target=update_event, args=(cr,uid, vals, cal_type, blmind_user, event.blmind_event_id))
                thr.start()
        else:
            cr.close()
        return res
    
    def unlink(self, cr, uid, ids, context=None):
        if not context:
            context = {}
        res = super(crm_meeting, self).unlink(cr, uid, ids, context)
        blimind_cal = self.pool.get('bluemind.calendar')
        dbname = cr.dbname
        db = sql_db.db_connect(dbname)
        cr = db.cursor()
        cal_type = self._is_auto_sync(cr, uid, context)
        res_user = self.pool.get('res.users').browse(cr, uid, [uid])[0]
        blmind_user = res_user.bluemind_user_id or False
        if cal_type and blmind_user and not context.get('from_blmind',False):
            blmind_cal_ids = blimind_cal.search(cr, uid,[('oerp_event_id','in',ids),('calendar_type_id','=',cal_type.id)])
            for event in blimind_cal.browse(cr, uid, blmind_cal_ids):
                delete_event = blimind_cal.delete_event
                thr = threading.Thread(target=delete_event, args=(cr,uid, cal_type, blmind_user, event.blmind_event_id))
                thr.start()
        else:
            cr.close()
        return res
    
    def _is_auto_sync(self, cr, uid, context=None):
        res = False
        cal_type = self.pool.get('calendar.type').search(cr, uid, [('model','=',self._name)])
        if cal_type:
            cal_type_id = cal_type[0]
            res_users = self.pool.get('res.users').browse(cr, uid, [uid],context)
            for user in res_users:
                for cal_tp in user.user_calendar_ids:
                    if cal_tp.id==cal_type_id:
                        res = cal_tp
        return res
    
crm_meeting()

class project_task(osv.osv):
    _inherit = 'project.task' 
    def create(self, cr, uid, vals, context=None):
        if not context:
            context = {}
        res = super(project_task, self).create(cr, uid, vals, context)
        if res:
            dbname = cr.dbname
            db = sql_db.db_connect(dbname)
            cr = db.cursor()
            cal_type = self._is_auto_sync(cr, uid, context)
            res_user = self.pool.get('res.users').browse(cr, uid, [uid])[0]
            blmind_user = res_user.bluemind_user_id or False
            if cal_type and blmind_user and vals['date_deadline'] and not context.get('from_blmind',False):
                vals['id'] = res
                create_event = self.pool.get('bluemind.calendar').create_event
                thr = threading.Thread(target=create_event, args=(cr,uid,vals,cal_type,blmind_user))
                thr.start()
            else:
                cr.close()            
        return res
    
    def write(self, cr, uid, ids, vals, context=None):
        if not context:
            context = {}
        res = super(project_task, self).write(cr, uid, ids, vals, context)
        blimind_cal = self.pool.get('bluemind.calendar')
        dbname = cr.dbname
        db = sql_db.db_connect(dbname)
        cr = db.cursor()
        cal_type = self._is_auto_sync(cr, uid, context)
        res_user = self.pool.get('res.users').browse(cr, uid, [uid])[0]
        blmind_user = res_user.bluemind_user_id or False
        if cal_type and blmind_user and not context.get('from_blmind',False):
            blmind_cal_ids = blimind_cal.search(cr, uid,[('oerp_event_id','in',ids),('calendar_type_id','=',cal_type.id)])
            for event in blimind_cal.browse(cr, uid, blmind_cal_ids):
                update_event = blimind_cal.update_event
                thr = threading.Thread(target=update_event, args=(cr,uid, vals, cal_type, blmind_user, event.blmind_event_id))
                thr.start()
        else:
            cr.close() 
        return res
    
    def unlink(self, cr, uid, ids, context=None):
        if not context:
            context = {}
        res = super(project_task, self).unlink(cr, uid, ids, context)
        blimind_cal = self.pool.get('bluemind.calendar')
        dbname = cr.dbname
        db = sql_db.db_connect(dbname)
        cr = db.cursor()
        cal_type = self._is_auto_sync(cr, uid, context)
        res_user = self.pool.get('res.users').browse(cr, uid, [uid])[0]
        blmind_user = res_user.bluemind_user_id or False
        if cal_type and blmind_user and not context.get('from_blmind',False):
            blmind_cal_ids = blimind_cal.search(cr, uid,[('oerp_event_id','in',ids),('calendar_type_id','=',cal_type.id)])
            for event in blimind_cal.browse(cr, uid, blmind_cal_ids):
                delete_event = blimind_cal.delete_event
                thr = threading.Thread(target=delete_event, args=(cr,uid, cal_type, blmind_user, event.blmind_event_id))
                thr.start()
        else:
            cr.close() 
        return res
    
    def _is_auto_sync(self, cr, uid, context=None):
        res = False
        cal_type = self.pool.get('calendar.type').search(cr, uid, [('model','=',self._name)])
        if cal_type:
            cal_type_id = cal_type[0]
            res_users = self.pool.get('res.users').browse(cr, uid, [uid],context)
            for user in res_users:
                for cal_tp in user.user_calendar_ids:
                    if cal_tp.id==cal_type_id:
                        res = cal_tp
        return res
    
project_task()

class hr_holidays(osv.osv):
    _inherit = 'hr.holidays'  
    def create(self, cr, uid, vals, context=None):
        if not context:
            context = {}
        res = super(hr_holidays, self).create(cr, uid, vals, context)
        if res:
            dbname = cr.dbname
            db = sql_db.db_connect(dbname)
            cr = db.cursor()
            cal_type = self._is_auto_sync(cr, uid, context)
            res_user = self.pool.get('res.users').browse(cr, uid, [uid])[0]
            blmind_user = res_user.bluemind_user_id or False
            if vals['name'] and cal_type and blmind_user and not context.get('from_blmind',False):
                vals['id'] = res
                create_event = self.pool.get('bluemind.calendar').create_event
                thr = threading.Thread(target=create_event, args=(cr,uid,vals,cal_type,blmind_user))
                thr.start()
            else:
                cr.close()   
        return res
    
    def write(self, cr, uid, ids, vals, context=None):
        if not context:
            context = {}
        res = super(hr_holidays, self).write(cr, uid, ids, vals, context)
        blimind_cal = self.pool.get('bluemind.calendar')
        dbname = cr.dbname
        db = sql_db.db_connect(dbname)
        cr = db.cursor()
        cal_type = self._is_auto_sync(cr, uid, context)
        res_user = self.pool.get('res.users').browse(cr, uid, [uid])[0]
        blmind_user = res_user.bluemind_user_id or False
        if cal_type and blmind_user and not context.get('from_blmind',False):
            blmind_cal_ids = blimind_cal.search(cr, uid,[('oerp_event_id','in',ids),('calendar_type_id','=',cal_type.id)])
            for event in blimind_cal.browse(cr, uid, blmind_cal_ids):
                update_event = blimind_cal.update_event
                thr = threading.Thread(target=update_event, args=(cr,uid, vals, cal_type, blmind_user, event.blmind_event_id))
                thr.start()
        else:
            cr.close()   
        return res
    
    def unlink(self, cr, uid, ids, context=None):
        if not context:
            context = {}
        res = super(hr_holidays, self).unlink(cr, uid, ids, context)
        blimind_cal = self.pool.get('bluemind.calendar')
        dbname = cr.dbname
        db = sql_db.db_connect(dbname)
        cr = db.cursor()
        cal_type = self._is_auto_sync(cr, uid, context)
        res_user = self.pool.get('res.users').browse(cr, uid, [uid])[0]
        blmind_user = res_user.bluemind_user_id or False
        if cal_type and blmind_user and not context.get('from_blmind',False):
            blmind_cal_ids = blimind_cal.search(cr, uid,[('oerp_event_id','in',ids),('calendar_type_id','=',cal_type.id)])
            for event in blimind_cal.browse(cr, uid, blmind_cal_ids):
                delete_event = blimind_cal.delete_event
                thr = threading.Thread(target=delete_event, args=(cr,uid, cal_type, blmind_user, event.blmind_event_id))
                thr.start()
        else:
            cr.close()   
        return res
    
    def _is_auto_sync(self, cr, uid, context=None):
        res = False
        cal_type = self.pool.get('calendar.type').search(cr, uid, [('model','=',self._name)])
        if cal_type:
            cal_type_id = cal_type[0]
            res_users = self.pool.get('res.users').browse(cr, uid, [uid],context)
            for user in res_users:
                for cal_tp in user.user_calendar_ids:
                    if cal_tp.id==cal_type_id:
                        res = cal_tp
        return res
    
hr_holidays()
